/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.example.websale;

import java.util.Collection;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.bonita.example.AbstractExample;
import org.ow2.bonita.facade.QueryDefinitionAPI;
import org.ow2.bonita.facade.QueryRuntimeAPI;
import org.ow2.bonita.facade.RuntimeAPI;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.runtime.var.Enumeration;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.util.AccessorUtil;
import org.ow2.bonita.util.BonitaException;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.novabpm.util.Misc;

/**
 * @author "Pierre Vigneras", "Marc Blachon", "Charles Souillard, "Miguel Valdes"
 */
public class WebSale extends AbstractExample {

  private static final Logger LOG = Logger.getLogger(WebSale.class.getName());

  private String userDecision = "reject";

  @Override
  public String getProcessId() {
    return "WebSale";
  }
  @Override
  public void afterInstantiate(final ProcessInstanceUUID instanceUUID,
      final ProcessDefinitionUUID processUUID,
      final RuntimeAPI runtimeAPI, final QueryRuntimeAPI queryRuntimeAPI,
      final QueryDefinitionAPI queryDefinitionAPI) throws BonitaException {

    Collection<ActivityInstance< TaskInstance >> activities = queryRuntimeAPI.getTaskList(instanceUUID, TaskState.READY);

    while (!activities.isEmpty()) {
      if (activities.isEmpty()) {
        throw new BonitaRuntimeException("No task found? Bad User?");
      }
      for (ActivityInstance< TaskInstance > activity : activities) {
        final TaskUUID taskUUID = activity.getBody().getUUID();
        final String activityId = activity.getActivityId();
        if (LOG.isLoggable(Level.INFO)) {
          LOG.info(Misc.LINE_SEPARATOR 
              + "***** Starting task associated to activity: " + activityId + " *****" 
              + Misc.LINE_SEPARATOR);
        }
        runtimeAPI.startTask(taskUUID, true);
        if (activity.getActivityId().equals("SalesReview")) {
          Enumeration currentEnum = (Enumeration)queryRuntimeAPI.getActivityInstanceVariable(activity.getUUID(), "decision");
          Enumeration newEnum = new Enumeration(currentEnum.getPossibleValues(),userDecision);          
          runtimeAPI.setActivityInstanceVariable(activity.getUUID(), "decision", newEnum);
        }

        if (LOG.isLoggable(Level.INFO)) {
          LOG.info(Misc.LINE_SEPARATOR + "***** Finishing task associated to activity: " + activityId + " *****"
              + Misc.LINE_SEPARATOR);
        }

        runtimeAPI.finishTask(taskUUID, true);
        if (userDecision.equals("moreinfo")) {
          // if we moved to moreinfo we set userDecision to reject to avoid infinite loop
          userDecision = "reject";
        }
        if (LOG.isLoggable(Level.INFO)) {
          LOG.info(Misc.LINE_SEPARATOR 
              + "***** Task associated to activity: " + activityId + " finished. *****"
              + Misc.LINE_SEPARATOR);
        }
      }
      activities = queryRuntimeAPI.getTaskList(instanceUUID, TaskState.READY);      
    }
  }

  public void setDecision(final String decision) {
    userDecision = decision;
  }

  public static void main(final String[] args) throws Exception {
    if (args == null || args.length != 2) {
      LOG.severe("Usage: java " + WebSale.class.getName() + " barFile <accepted>");
      LOG.severe("\t<accepted>: whether the request is approved "
          + "('grant'), rejected ('reject' )or more info is required ('moreinfo')");
      throw new BonitaRuntimeException("Please read the above usage");
    }
    final String barFile = args[0];
    String decision = args[1];

    if (!args[1].equals("grant") && !args[1].equals("reject") && !args[1].equals("moreinfo")) {
      decision = "reject";
    }

    final WebSale websale = new WebSale();
    websale.login();
    websale.setDecision(decision);
    websale.execute(
        AccessorUtil.getAPIAccessor().getRuntimeAPI(),
        AccessorUtil.getAPIAccessor().getQueryRuntimeAPI(),
        AccessorUtil.getAPIAccessor().getManagementAPI(),
        AccessorUtil.getAPIAccessor().getQueryDefinitionAPI(),
        barFile, null);
    websale.logout();
  }

}
